Import os
Import mojo

Class MStream
	
	Field CurrentLine:Int = 0
	Field Lines:String[]
	
	Method New(url:String)
		Lines = os.LoadString(url).Split("~r")
	End

	Method EOF:Bool()
		If(CurrentLine<=Lines.Length-1) Return False
		Return True
	End
	
	Method ReadLine:String()
		CurrentLine += 1
		If(CurrentLine<=Lines.Length)		
			Return Lines[CurrentLine-1].Trim()
		Endif 
		Return ""
	End
	
End

Function EOF(stream:MStream)
	Return stream.EOF()
End

Const ALPHABLEND = 0
Const AlphaBLEND = 0
Const MASKBLEND = 0

'Drawing command classes
Global DrawingCommands:=New List<MDrawingCommand>()
Global Rotation:Float = 0
Global ScaleX:Float = 1
Global ScaleY:Float = 1
Global autoMidHandle:Bool

Class MDrawingCommand

	Method Action()
	End Method

End Class

Class DrawTextCommand extends MDrawingCommand
	
	Field Text:String
	Field X:Float
	Field y:Float
	
	Method New(text:String,x:Float,y:Float)
		Text = text
		X = x
		y = y
	End

	Method Action()
		DrawText(Text,X,y)
	End Method
	
End

Class ColourCommand Extends MDrawingCommand
	
	Field Red:Float
	Field Green:Float
	Field Blue:Float
	Field IsSetColor:Bool
	
	Method New(red:Float,green:Float,blue:Float,isSetColor:Bool)
		Red = red
		Green = green
		Blue = blue
		IsSetColor = isSetColor
	End

	Method Action()
		if(IsSetColor)
			SetColor Red,Green,Blue
		Else
			SetClsColor Red,Green,Blue
		EndIf
	End Method
	
End Class

Class ClsCommand extends MDrawingCommand
	Method Action()
		Cls
	End
End

Class RectCommand Extends MDrawingCommand
	
	Field X:Float
	Field y:Float
	Field Width:Float
	Field Height:Float
	Field IsOval:Bool = False
	
	Method New(x:Float,y:Float,width:Float=1,height:Float=1,isOval:Bool=False)
		X = x
		y = y
		Width = width
		Height = height
		IsOval = isOval
	End Method

	Method Action()
		If(IsOval=False)
			DrawRect(X,y,Width,Height)
		Else
			DrawOval(X,y,Width,Height)
		Endif
	End Method

End Class

Class LineCommand Extends MDrawingCommand
	
	Field X:Float
	Field y:Float
	Field X2:Float
	Field Y2:Float
	Field DrawLastPixel:Bool = True 'Note - currently not handled, only there for legacy support
	
	Method New(x:Float,y:Float,x2:Float,y2:Float,drawLastPixel:Bool=True)
		X = x
		y = y
		X2 = x2
		Y2 = y2
		DrawLastPixel = drawLastPixel
	End Method
	
	Method Action()
		DrawLine(X,y,X2,Y2)
	End Method

End Class

Class PolyCommand Extends MDrawingCommand

	Field XY:Float[]

	Method New(xy:Float[])
		XY = xy
	End Method
	
	Method Action()
		DrawPoly(xy)
	End Method

End Class

Class ImageCommand Extends MDrawingCommand
	
	Field Image:Image
	Field X:Float
	Field Y:Float
	Field Frame:Int
	
	Field W:Float
	Field H:Float
	Field IsImageRect:Bool = False
	Field IsTileImage:Bool = false
	
	Method New(image:Image,x:Float,y:Float,frame,isTileImage:Bool=false)
		Image = image
		X = x
		Y = y
		Frame = frame
		IsTileImage = isTileImage
	End Method
	
	Method New(image:Image,x:Float,y:Float,w:Float,h:Float,frame)
		Image = image
		X = x
		Y = y
		W = w
		H = h
		IsImageRect = True
		Frame = frame
	End Method
	
	Method Action()
	
		If(IsImageRect=False)
		
			if(IsTileImage=True)
			
				'Todo: Handle Scale, Origin and Rotation
			
				Local startx:Float = X
				While(startx>0)
					startx -= Image.Width
				Wend
				
				Local starty:Float = Y
				While(starty>0)
					starty -= Image.Width
				Wend
			
				Local imagex = startx
				Local imagey = starty
				While(imagex<640)
					While(imagey<640)
						DrawImage(Image,imagex,imagey,Rotation,ScaleX,ScaleY,Frame)
						imagey += Image.Height
					Wend
					imagex += Image.Width
					imagey = starty
				Wend
				
			else
			
				DrawImage(Image,X,Y,Rotation,ScaleX,ScaleY,Frame)
				
			EndIf			
			
		Else
			DrawImageRect(Image,X,Y,0,0,W,H,Rotation,ScaleX,ScaleY,Frame)
		Endif
	End Method

End Class

Class AlphaCommand extends MDrawingCommand
	
	Field Alpha:Float

	Method New(alpha:Float)
		Alpha = alpha
	End
	
	Method Action()
		SetAlpha Alpha
	End

End

Class BlendCommand extends MDrawingCommand
	Field Value
	
	Method New(value)
		Value = value
	End Method
	
	Method Action()
		SetBlend Value
	End
End

Class SetScaleCommand extends MDrawingCommand

	Field SX:Float
	Field SY:Float

	Method New(sx:Float,sy:Float)
		SX = sx
		SY = sy
	End
	
	Method Action()
		ScaleX = SX
		ScaleY = SY
	End
	
End

Class SetRotationCommand extends MDrawingCommand

	Field Rot:Float

	Method New(rot:Float)
		Rot = rot
	End
	
	Method Action()
		Rotation = Rot
	End
End

'Drawing command functions
Function MDrawText(text:String,x:Float,y:float)
	DrawingCommands.AddLast(New DrawTextCommand(text,x,y))
End

Function MLoadImage:Image(url:string,flags=-1 )
	
	url = url.Replace("\","/")

	if flags=-1
		flags = Image.DefaultFlags
		if(autoMidHandle=True) Then flags = Image.MidHandle
	EndIf

	Local image:Image =  LoadImage(url,1,flags)
	if(image=null) Print "Error: Cannot load image " + url
	Return image
	
End Function

Function MLoadAnimImage:Image( url:string,cell_width,cell_height,first_cell,cell_count,flags=-1 )

	url = url.Replace("\","/")

	if flags=-1
		flags = Image.DefaultFlags
		if(autoMidHandle=True) Then flags = Image.MidHandle
	EndIf
	
	Local image:Image = LoadImage( url, cell_width, cell_height, cell_count, flags )
	if(image=null) Print "Error: Cannot load image " + url
	Return image
	
End Function

Function AutoMidHandle(value:Bool)
	autoMidHandle = value
End

Function MCls()
	DrawingCommands.AddLast(New ClsCommand())
End Function

Function SetClsColor(red:Float,green:Float,blue:Float)
	DrawingCommands.AddLast(New ColourCommand(red,green,blue,False))
End Function

Function MSetColor(red:Float,green:Float,blue:Float)
	DrawingCommands.AddLast(New ColourCommand(red,green,blue,True))
End Function

Function Plot(X:Float,y:Float)
	DrawingCommands.AddLast(New RectCommand(X,y))
End Function

Function MDrawRect(X:Float,y:Float,Height:Float,Width:Float)
	DrawingCommands.AddLast(New RectCommand(X,y,Height,Width))
End Function

Function MDrawLine(X:Float,y:Float,X2:Float,Y2:Float,DrawLastPixel:Bool=True)
	DrawingCommands.AddLast(New LineCommand(X,y,X2,Y2,DrawLastPixel))
End Function

Function MDrawOval(X:Float,y:Float,Height:Float,Width:Float)
	DrawingCommands.AddLast(New RectCommand(X,y,Height,Width,True))
End Function

Function MDrawPoly(xy:Float[])
	DrawingCommands.AddLast(New PolyCommand(xy))
End Function

Function MDrawImage(image:Image,x:Float,y:Float,frame:Int=0)
	DrawingCommands.AddLast(New ImageCommand(image,x,y,frame,false))
End Function

Function MDrawImageRect(image:Image,x:Float,y:Float,w:Float,h:Float,frame=0)
	DrawingCommands.AddLast(New ImageCommand(image,x,y,w,h,frame))
End Function

Function TileImage(image:Image,x:Float,y:Float,frame:Int=0)
	DrawingCommands.AddLast(New ImageCommand(image,x,y,frame,True))
End Function

Function MSetAlpha(alpha:Float)
	DrawingCommands.AddLast(New AlphaCommand(alpha))
End

Function MSetBlend(blend:Int)
	DrawingCommands.AddLast(New BlendCommand(blend))
End

Function SetScale(x:Float,y:Float)
	DrawingCommands.AddLast(New SetScaleCommand(x,y))
End

Function SetRotation(value:Float)
	DrawingCommands.AddLast(New SetRotationCommand(value))
End